from setuptools import setup, find_packages

setup(
    name='dwj_tools', # 项目的名称,pip3 install get-time
    version='0.0.4', # 项目版本 
    author='丁文杰', # 项目作者 
    author_email='359582058@qq.com', # 作者email 
    url='https://github.com/Coxhuang/get_time', # 项目代码仓库
    description='私人', # 项目描述 
    packages=find_packages(), # 包名 
    install_requires=[],
    entry_points={
        'console_scripts': [
            'get_time=get_time:get_time', # 使用者使用get_time时,就睡到get_time项目下的__init__.py下执行get_time函数
            'get_timestamp=get_time:get_timestamp',
            'timestamp_to_str=get_time:timestamp_to_str',
            'str_to_timestamp=get_time:str_to_timestamp',
        ]
    } # 重点
)

